#!/usr/bin/env python3


"""
.git/
- zip and split the history

./
- zip and split the history

./**/*.x
- compress x
"""
class niceghapi(object):
	def __init__(self):
		self.cur_status = None
		self.now = None
		self.resetdate = None

	@property
	def status(self):
		# curl -I https://api.github.com/users/octocat|grep x-ratelimit-reset
		self.cur_status, self.now = requests.get("https://api.github.com/users/octocat").headers, datetime.datetime.now()
		return {
			'Reset': self.cur_status['X-RateLimit-Reset'],
			'Used': self.cur_status['X-RateLimit-Used'],
			'Total': self.cur_status['X-RateLimit-Limit'],
			'Remaining': self.cur_status['X-RateLimit-Remaining'],
			'RemainingDate':datetime.datetime.fromtimestamp(int(self.cur_status['X-RateLimit-Reset'])),
			'WaitFor':datetime.datetime.fromtimestamp(int(self.cur_status['X-RateLimit-Reset'])) - self.now,
			'WaitForSec':(datetime.datetime.fromtimestamp(int(self.cur_status['X-RateLimit-Reset'])) - self.now).seconds,
			'WaitForNow':lambda :(datetime.datetime.fromtimestamp(int(self.cur_status['X-RateLimit-Reset'])) - datetime.datetime.now()).seconds,
		}

	@property
	def timing(self):
		import time
		if not hasattr(self, 'remaining') or self.remaining is None:
			stats = self.status
			print(stats)
			self.remaining = int(stats['Remaining'])
			self.wait_until = stats['WaitForNow']
			self.resetdate = stats['RemainingDate']
			self.timing
		elif self.remaining >= 10:
			self.remaining = self.remaining - 1
		else:
			print("Waiting until: {0}".format(self.resetdate))
			pause.until(self.resetdate)
			delattr(self, 'remaining')
			delattr(self, 'wait_until')
		return




class RepoGather(object)
    def __init__(self, url:str, token:str):
        self.url = mystring.string(url)
        req.headers.update({'Authorization': 'token ' + token})
        self.clone_url = url.repsies('github.com/','https://','http://', '.git')
        self.clone_url = mystring.string('https://api.github.com/repos/' + self.clone_url+'/contents')

    def clone(self):
        clone(self.clone_url, self.url, self.url.split('/')[-1], None)

    def handleGit(self):
        if not os.path.exists('.git/'):
            return

    def __enter__(self):
        pass

    def __exit__(self, exc_type, exc_val, exc_tb):
        return self
    

GRepo_Saving_Progress_Lock = threading.Lock()

class GRepo_Pod(object):
	def __init__(self, token:str=None,num_processes:int = None, delete_paths:bool=False):
		self.token = token
        if "GH_TOKEN" not in os.environ:
			self.login()

		self.g = Github(self.token)
		self.processor = mystring.MyThreads(10)
		self.processed_paths = queue.Queue()
		setattr(self.processed_paths, 'lock', threading.Lock())

		self.current_repo_itr = None
		self.total_repo_len = None
		self.num_processes = num_processes

		def appr(string: mystring.string):
			with open("mapping_file_{0}.csv".format(string.tobase64()), "a+") as writer:
				writer.write(string)
		self.appr = appr
		self.api_watch = niceghapi()
		self.delete_paths = delete_paths
		self.query_string = None
		self.tracking_repos = None
		self.tracking_name = None
		asyncio.run(self.handle_history())

	@property
	def localfilename(self):
		if self.tracking_name is None:
			self.tracking_name = mystring.string("query_progress_{0}.csv".format(
				mystring.string("{query_string}".format(query_string=self.query_string)).tobase64())
			)
		return self.tracking_name


	@property
	def repos(self):
		if self.tracking_repos is None:
			self.tracking_repos = []
			if os.path.exists(self.localfilename):
				with open(self.localfilename, "r") as reader:
					for line in reader:
						ProjectItr, ProjectURL, ProjectScanned = line.split(",")
						if ProjectScanned == "false":
							self.tracking_repos.append(ProjectURL)
			else:
				self.tracking_repos = [x.clone_url for x in self.g.search_repositories(query=self.query_string)]
		return self.tracking_repos

	def save(self, current_project_url:str=None):
		#Make Thread Safe
		with GRepo_Saving_Progress_Lock:
			if not os.path.exists(self.localfilename):
				with open(self.localfilename, "w+") as writer:
					writer.write("ProjectItr,ProjectURL,ProjectScanned\n")
					for proj_itr, proj in enumerate(self.repos):
						writer.write("{0},{1},false\n".format(proj_itr, proj))

			if current_project_url is not None:
				found = False
				with finput(self.localfilename, inplace=True) as reader:
					for line in reader:
						if not found and current_project_url in line:
							line = line.replace("false", "true")
						print(line, end='')
		return

	@property
	def timing(self):
		self.api_watch.timing

		extra_rate_limiting = self.g.get_rate_limit()
		if hasattr(extra_rate_limiting, "search"):
			search_limits = getattr(extra_rate_limiting, "search")
			if search_limits.remaining < 2:
				print("Waiting until: {0}".format(search_limits.reset))
				pause.until(search_limits.reset)
	
	def repair(self,path,create:bool=True):
		if self.delete_paths and os.path.exists(path):
			os.system("yes|rm -r "+str(path))
		if create:
			os.makedirs(path, exist_ok=True)

	def __call__(self, search_string:str):
		self.timing
		search_string = mystring.string(search_string)

		def process_prep(repo_itr:int, repo_clone_url:str, search_string:str, appr:Callable, fin_queue:queue.Queue):
			self.query_string = search_string
			def process():
				name = mystring.string("ITR>{0}_URL>{1}_STR>{2}\n".format(
					repo_itr, repo_clone_url, search_string
				))
				repo_dir = "repo_" + str(name.tobase64())
				results_dir = "results_" + str(name.tobase64())

				self.repair(repo_dir, create=False)
				self.repair(results_dir)

				appr(mystring.string(name.replace(',',';').replace('_',',').strip()))
				fin_queue.put(repo_clone_url)

			return process

		if len(self.repos) > 0:
			for repo_itr, repo_url in enumerate(self.repos):
				self.processor += process_prep(repo_itr, repo_url, search_string, self.appr, self.processed_paths)
				self.current_repo_itr = repo_itr
		else:
			print("No Repos Found")

	def login(self):
		os.environ['GH_TOKEN'] = self.token
		with suppress(Exception):
			with open("~/.bashrc", "a+") as writer:
				writer.write("GH_TOKEN={0}".format(self.token))

	@property
	def complete(self):
		return self.total_repo_len == self.current_repo_itr and self.processor.complete

	async def handle_history(self):
		while not self.complete:
			# Get up to 5 strings from the queue
			paths,num_waiting = [], 5
			while len(paths) < num_waiting:
				try:
					path = self.processed_paths.get()
					paths.append(path)
				except queue.Empty:
					time.sleep(10)
					num_waiting -= 1

			# Process the strings
			for path in paths:
				mystring.string("git add {0}".format(path)).exec()
			mystring.string("git commit -m \"Added multiple paths\"").exec()
			mystring.string("git push").exec()
			for path in paths:
				mystring.string("yes|rm -r {0}".format(path)).exec()

			for path in paths:
				self.save(path)
